@@ -3,7 +3,7 @@ package Thread::Tie::Thread;
 # Make sure we have version info for this module
 # Make sure we do everything by the book from now on
 
-our $VERSION : unique = '0.02';
+our $VERSION : unique = '0.03';
 use strict;
 
 # Make sure we can do threads
@@ -15,15 +15,15 @@ use threads::shared ();
 use Storable ();
 
 # Make sure the freeze and thaw routines are in memory
-# Create frozen version of empty list
-# Save the iced signature
+# Create the iced signature
 
 Storable::thaw( Storable::freeze( [] ) );
-my $iced = Storable::freeze( [] );
-$iced = unpack( 'l',$iced );
+my $iced = unpack( 'l',Storable::freeze( [] ) );
 
+# Thread local list of tied objects
 # Clone detection logic
 
+my @OBJECT;
 my $CLONE = 0;
 
 # Satisfy -require-
@@ -85,6 +85,28 @@ sub thread { threads->object( shift->{'tid'} ) } #thread
 
 sub tid { shift->{'tid'} } #tid
 
+#---------------------------------------------------------------------------
+#  IN: 1 instantiated object
+
+sub shutdown {
+
+# Obtain the object
+# Return if we're not in the originating thread
+# Return now if already shut down
+
+    my $self = shift;
+    return if $self->{'CLONE'} != $CLONE;
+    return unless defined( $self->{'tid'} );
+
+# Shut the thread down
+# Wait for it to be actually finished
+# Mark the thread as shut down
+
+    $self->_handle;
+    $self->thread->join;
+    undef( $self->{'tid'} );
+} #shutdown
+
 #---------------------------------------------------------------------------
 
 # internal methods
@@ -107,7 +129,7 @@ sub _handle {
     my ($control,$data) = @$self{qw(control data)};
     my $frozen = _freeze( \@_ );
 
-# Initialize the counter
+# Initialize the tries counter
 # While we haven't got access to the handler
 #  Give up this timeslice if we tried this before
 #  Wait for access to the belt
@@ -148,21 +170,25 @@ sub _handler {
 # Obtain the object
 # Obtain the references to the fields that we need
 # Subroutine to execute
-# Thread local list of tied objects
 # Ordinal number of object to which it is tied
 
     my $self = shift;
     my ($control,$data) = @$self{qw(control data)};
     my $sub;
-    my @object;
     my $ordinal;
 
 # Initialize general dispatch
+
+    my %dispatch = (
+     EVAL    => \&doEVAL,
+     UNTIE   => \&doUNTIE,
+     USE     => \&doUSE,
+    );
+
 # Local copy of object to use
 # Local copy of code to execute
 # Frozen copy of no values
 
-    my %dispatch;
     my $object;
     my $code;
     my $undef = _freeze( [undef] );
@@ -192,44 +218,52 @@ sub _handler {
 
 #  Obtain the name of the subroutine to execute
 #  Obtain the ordinal number of the object to execute + data to be sent
-#  If we have a tie action
-#   If it is a known tie method
-#    Perform the appropriate tieing subroutine
-#   Else
-#    Die, we don't know how to handle this
 
-        {no strict 'refs';
-	 $sub = $$control;
-         ($ordinal,@_) = _thaw( $$data );
-         if ($sub =~ m#^(.*)::(TIE\w+)$#) {
-             if ($sub = $tie_dispatch{ $2 }) {
-                 $object[$ordinal] = $sub->( $1,@_ );
-             } else {
-                 die "Don't know how to tie with $sub";
-             }
+        $sub = $$control;
+        ($ordinal,@_) = _thaw( $$data );
 
-#  Elsif there is an object for this ordinal number, saving object on the fly
+#  If we have an object, obtaining local copy of object on the fly
 #   If we have a code reference for this method, saving it on the fly
 #   Elseif we haven't checked before
 #    Normalize the subroutine name
 #    Obtain a code reference for this method on this object if here is one
 #   Call the method with the right object and save result
 
-         } elsif ($object = $object[$ordinal]){
-             if ($code = $dispatch{$sub}) {
-             } elsif( !exists( $dispatch{$sub} ) ) {
-                 $sub =~ s#^.*::##;
-                 $code = $dispatch{$sub} = $object->can( $sub );
-             }
-             $$data = $code ? _freeze( [$code->( $object,@_ )] ) : $undef;
+        if ($object = $OBJECT[$ordinal]) {
+            if ($code = $dispatch{$sub}) {
+            } elsif( !exists( $dispatch{$sub} ) ) {
+                (my $localsub = $sub) =~ s#^.*::##;
+                $code = $dispatch{$sub} = $object->can( $localsub );
+            }
+            $$data = $code ? _freeze( [$code->( $object,@_ )] ) : $undef;
 
-#  Else (we don't have an object yet)
-#   Just call the sub routine
+#  Elseif we have a tie action
+#   If it is a known tie method
+#    Perform the appropriate tieing subroutine
+#   Else (unknown tie method)
+#    Die, we don't know how to handle this
 
-         } else {
-             $$data = _freeze( [$sub->( @_ )] )
-         }
-	} #no strict refs
+        } elsif ($sub =~ m#^(.*)::(TIE\w+)$#) {
+            if ($sub = $tie_dispatch{ $2 }) {
+                $OBJECT[$ordinal] = $sub->( $1,@_ );
+            } else {
+                die "Don't know how to TIE with $sub";
+            }
+
+#  Elseif we're attempting to destroy without an object
+#   Just set an undefined results (assume it is DESTROY after untie()
+#  Elseif it is a known subroutine that is allowed
+#   Execute the action, assume it's a special startup function
+#  Else
+#   Die now, this is strange!
+
+        } elsif ($sub =~ m#DESTROY$#) {
+            $$data = $undef;
+        } elsif ($code = $dispatch{$sub}) {
+            $$data = $code->( undef,@_ );
+        } else {
+            die "Attempting to $sub without an object at $ordinal\n";
+        }
 
 #  Mark the data to be ready for usage
 #  Wait until the caller has taken it
@@ -310,32 +344,57 @@ sub DESTROY {
 # Obtain the object
 # Return if we're not in the originating thread
 # Shut the thread down
-# And wait for it to be actually finished
 
     my $self = shift;
     return if $self->{'CLONE'} != $CLONE;
-    $self->_handle;
-    $self->thread->join;
+    $self->shutdown;
 } #DESTROY
 
 #---------------------------------------------------------------------------
-#  IN: 1 module to load
-#      2..N any parameters to import
+#  IN: 1 object (ignored)
+#      2 code to eval
 
-sub USE {
+sub doEVAL { eval( $_[1] ) } #doEVAL
 
+#---------------------------------------------------------------------------
+#  IN: 1 object
+#      2 ordinal number of object to remove
+
+sub doUNTIE {
+
+# Obtain the object
+# If we can destroy the object, obtaining code ref on the fly
+#  Perform whatever needs to be done to destroy
+# Kill all references to the variable
+
+    my $object = shift;
+    if (my $code = $object->can( 'DESTROY' )) {
+        $code->( $object );
+    }
+    undef( $OBJECT[shift] );
+} #doUNTIE
+
+#---------------------------------------------------------------------------
+#  IN: 1 object (ignored)
+#      2 module to load
+#      3..N any parameters to import
+
+sub doUSE {
+
+# Remove object
 # Obtain the class
 # Create a copy for the filename
 # Make sure we have a correct filename
 # Load the module file
 # Execute import routine (if any)
 
+    shift;
     my $class = shift;
     my $file = $class;
     $file =~ s#::#/#g; $file .= '.pm';
     require $file;
     $class->import( @_ );
-} #USE
+} #doUSE
 
 #---------------------------------------------------------------------------
 
@@ -345,9 +404,64 @@ __END__
 
 Thread::Tie::Thread - create threads for tied variables
 
+=head1 SYNOPSIS
+
+    use Thread::Tie; # use as early as possible for maximum memory savings
+
+    my $tiethread = Thread::Tie::Thread->new;
+    tie stuff, 'Thread::Tie', {thread => $thread};
+
+    my $tid = $tiethread->tid;        # thread id of tied thread
+    my $thread = $tiethread->thread;  # actual "threads" thread
+    $tiethread->shutdown;             # shut down specific thread
+
 =head1 DESCRIPTION
 
-Helper class for L<Thread::Tie>.  See documentation there.
+                  *** A note of CAUTION ***
+
+ This module only functions on Perl versions 5.8.0 and later.
+ And then only when threads are enabled with -Dusethreads.  It
+ is of no use with any version of Perl before 5.8.0 or without
+ threads enabled.
+
+                  *************************
+
+The Thread::Tie::Thread module is a helper class for the L<Thread::Tie>
+module.  It is used to create the thread in which the actual code, to which
+variables are tied with the Thread::Tie class, is located.
+
+Please see the documentation of the L<Thread::Tie> module for more
+information.
+
+=head1 CLASS METHODS
+
+There is only one class method.
+
+=head2 new
+
+ my $tiethread = Thread::Tie::Thread->new;
+
+The "new" class method returns an instantiated object that can be specified
+with the "thread" field when tie()ing a variable.
+
+=head1 OBJECT METHODS
+
+The following object methods are available for the instantiated
+Thread::Tie::Thread object.
+
+=head2 tid
+
+ my $tid = $tiethread->tid;
+
+The "tid" object method returns the thread id of the actual L<threads>
+thread that is being used.
+
+=head2 thread
+
+ my $thread = $tiethread->thread;
+
+The "thread" object method returns the actual L<threads> thread object that
+is being used.
 
 =head1 AUTHOR
 
@@ -363,6 +477,6 @@ modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<Thread::Tie>.
+L<Thread::Tie>, L<threads>.
 
 =cut
